@@ -3,7 +3,7 @@
  *  Definition of optimizing macros. */
 
 /*
- *  GiNaC Copyright (C) 1999-2018 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2023 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -33,9 +33,4 @@
 #define attribute_deprecated
 #endif
 
-#ifdef _MSC_VER
-#define __func__ __FUNCTION__
-#define __alignof__ __alignof
-#endif
-
 #endif // ndef GINAC_COMPILER_DEP_H
