@@ -3,7 +3,7 @@
  *  Collection of all flags used through the GiNaC framework. */
 
 /*
- *  GiNaC Copyright (C) 1999-2018 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2023 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -182,7 +182,15 @@ public:
 		 *  linear systems.  In contrast to division-free elimination it only
 		 *  has a linear expression swell.  For two-dimensional systems, the
 		 *  two algorithms are equivalent, however. */
-		bareiss
+		bareiss,
+		/** Markowitz-ordered Gaussian elimination. Same as the usual
+		 *  Gaussian elimination, but with additional effort spent on
+		 *  selecting pivots that minimize fill-in. Faster than the
+		 *  methods above for large sparse matrices (particularly with
+		 *  symbolic coefficients), otherwise slightly slower than
+		 *  Gaussian elimination.
+		 */
+		markowitz
 	};
 };
 
