@@ -3,7 +3,7 @@
  *  Interface to parser context. */
 
 /*
- *  GiNaC Copyright (C) 1999-2011 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2023 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -88,7 +88,20 @@ typedef ex (*reader_func)(const exvector& args);
  *       function pointer!! The unsigned has to correspond to the serial number of
  *       the defined GiNaC function.
  */
-typedef std::map<prototype, reader_func> prototype_table;
+class PrototypeLess
+{
+public:
+	bool operator()(const prototype& p1, const prototype& p2) const
+	{
+		int s = p1.first.compare(p2.first);
+		if (s == 0) {
+			if ((p1.second == 0) || (p2.second == 0)) return false;
+			return p1.second < p2.second;
+		}
+		return s < 0;
+	}
+};
+typedef std::map<prototype, reader_func, PrototypeLess> prototype_table;
 
 /**
  * Default prototype table.
